#!/usr/bin/env node
'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

function _interopNamespace(e) {
  if (e && e.__esModule) { return e; } else {
    var n = {};
    if (e) {
      Object.keys(e).forEach(function (k) {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () {
            return e[k];
          }
        });
      });
    }
    n['default'] = e;
    return n;
  }
}

var fs = require('fs');
var fs__default = _interopDefault(fs);
var path = require('path');
var path__default = _interopDefault(path);
var cac = require('cac');
var index = require('./index-9fffc052.js');

class PrettyError extends Error {
    constructor(message) {
        super(message);
        this.name = this.constructor.name;
        if (typeof Error.captureStackTrace === 'function') {
            Error.captureStackTrace(this, this.constructor);
        }
        else {
            this.stack = new Error(message).stack;
        }
    }
}
function handlError(error) {
    if (error.loc) {
        console.error(index.$.bold(index.$.red(`Error parsing: ${error.loc.file}:${error.loc.line}:${error.loc.column}`)));
    }
    if (error.frame) {
        console.error(index.$.red(error.message));
        console.error(index.$.dim(error.frame));
    }
    else {
        if (error instanceof PrettyError) {
            console.error(index.$.red(error.message));
        }
        else {
            console.error(index.$.red(error.stack));
        }
    }
    process.exitCode = 1;
}

async function main() {
    const cli = cac.cac('tsup');
    cli
        .command('[...files]', 'Bundle files')
        .option('-d, --out-dir <dir>', 'Output directory', { default: 'dist' })
        .option('--format <format>', 'Bundle format, "cjs", "iife", "umd", "esm"', {
        default: 'cjs',
    })
        .option('--bundle', 'Bundle node_modules')
        .option('--dts', 'Generate declaration file')
        .option('--dts-bundle', 'Bundle types from node_modules')
        .option('--watch', 'Watch mode')
        .option('--define.* <value>', 'Define compile-time constants')
        .option('--external <name>', 'Mark specific packages as external (use with --bundle)')
        .option('--module-name <name>', 'Module name (with with --format umd)')
        .option('--jsxFactory <jsxFactory>', 'Name of JSX factory function', {
        default: 'React.createElement',
    })
        .option('--jsxFragment <jsxFragment>', 'Name of JSX fragment function', {
        default: 'React.Fragment',
    })
        .option('--inlineDynamicImports', 'Create a single bundle that inlines dynamic imports')
        .action(async (files, options) => {
        if (files.length === 0) {
            throw new PrettyError(`Missing input files, e.g. tsup src/index.ts`);
        }
        const { rollup, watch } = await Promise.resolve().then(function () { return _interopNamespace(require('rollup')); });
        const { createRollupConfigs, printSizes } = await Promise.resolve().then(function () { return require('./index.js'); });
        const rollupConfigs = await createRollupConfigs(files, options);
        if (options.watch) {
            const watcher = watch(rollupConfigs.map((config) => ({
                ...config.inputConfig,
                output: config.outputConfig,
            })));
            watcher.on('event', (event) => {
                console.log(event);
            });
        }
        else {
            const startTime = Date.now();
            await Promise.all(rollupConfigs.map(async (config) => {
                var _a;
                try {
                    const result = await rollup(config.inputConfig);
                    await result.write(config.outputConfig);
                }
                catch (err) {
                    console.error(`Failed with following plugins used: ${(_a = config.inputConfig.plugins) === null || _a === void 0 ? void 0 : _a.map((p) => p.name).join(', ')}`);
                    throw err;
                }
            }));
            printSizes();
            const endTime = Date.now();
            console.log(`Done in ${endTime - startTime}ms`);
        }
    });
    cli
        .command('run <file>', 'Bundle and execute a file', {
        allowUnknownOptions: true,
    })
        .option('--define.* <value>', 'Define compile-time constants')
        .action(async (file, options) => {
        const extraArgs = process.argv.slice(process.argv.indexOf(file) + 1);
        const { rollup } = await Promise.resolve().then(function () { return _interopNamespace(require('rollup')); });
        const { createRollupConfigs } = await Promise.resolve().then(function () { return require('./index.js'); });
        const { runCode } = await Promise.resolve().then(function () { return require('./run-e6893bf5.js'); });
        const [rollupConfig] = await createRollupConfigs([file], {
            define: options.define,
            outDir: 'dist',
            format: 'cjs',
        });
        const bundle = await rollup(rollupConfig.inputConfig);
        const { output } = await bundle.write(rollupConfig.outputConfig);
        runCode(path.join('dist', output[0].fileName), {
            args: extraArgs,
        });
    });
    cli.help();
    const pkgPath = path.join(__dirname, '../package.json');
    cli.version(JSON.parse(fs.readFileSync(pkgPath, 'utf8')).version);
    cli.parse(process.argv, { run: false });
    await cli.runMatchedCommand();
}
main().catch(handlError);
